/* LimePTR grid directive for UI directives for AngularJS 1.1.4
 *
 * Copyright (c) 2016 Lime PTR
 * Author: Shafaq Kazmi
 *
 * Problem:: igTreeGrid directives for AngularJS does not support two way data binding
 * For datasource update we need to rebind the whole data with igTreeGrid. Issue with this approaches are:
 * - Data binding requires rendering all rows again
 * - expand/collapse & scroll state doesn't preserve on update
 * Solution:: Enabling Updating, identify the change set and add/update/delete accordingly
 *
 * This directive does only following this
 * 1: Convert Flat data to Tree (igTreeGrid requires tree data)
 * 2: On data response, if there is a change then identify change type and perform action accordingly
 */
ptrApp.directive('limeGrid', function() {
	return {
		restrict: 'E',
		scope: {
			options: "=",
			gridId: "@"
		},
		replace: true,
		template: '<div ng-if="options" id="igniteGridId" ig-tree-grid="options" />',

		link: function(scope, element, attrs) {

			var gridId = scope.gridId,
				delay = 10,
				timeout,
				optionWatcher,
				gridElement,
				gridElementContainer,
				dataSource = [];

			optionWatcher = scope.$watch('options', function(data) {

				if (data) {
					optionWatcher(); //clear watch
					gridElement = $("#" + gridId);
					gridElementContainer = $("#" + gridId + "_displayContainer");
				}
			});

			scope.$watch('options.dataSource', function(treeData) {

				if (treeData && treeData.length > 0) {

					console.log("****************************** igTreeGrid Operation START ****************************");
					var startTime = new Date().getTime();
					var grid = gridElement.data('igTreeGrid'),
						treeDataHash = {},
						oldTreeDataHash = {};

					if (grid.options.dataSource.length == 0) { //grid is rendering for the 1st time
						grid.options.dataSource = treeData;
						grid.dataBind();

					} else if (!angular.equals(treeData, dataSource)) {

						flattenTreeToHashMap(treeData, -1, treeDataHash);
						flattenTreeToHashMap(dataSource, -1, oldTreeDataHash);
						updateVisibleRows(treeDataHash, oldTreeDataHash);
					}

					dataSource = treeData;

					console.log(new Date().getTime() - startTime + " ms");
					console.log("****************************** igTreeGrid Operation END ******************************");
				}
			});

			function updateFilterAndSortState() {

				var grid = gridElement.data('igTreeGrid'),
					expressions = grid.dataSource.settings.filtering.expressions;

				if (expressions.length > 0) {
					gridElement.igTreeGridFiltering("filter", expressions);
				}

				gridElement.igTreeGridSorting("sortMultiple");
			}

			function updateVisibleRows(treeDataHash, oldTreeDataHash) {

				var hasChange = false,
					getObjectWithoutChildren = function(o1) {

						var newObject = null;
						for (var key in o1) {
							if (key != "Children") {
								newObject = newObject || {};
								newObject[key] = o1[key];
							}
						}

						return newObject;
					};

				for (var key in oldTreeDataHash) {

					var newRecord = treeDataHash[key],
						oldRecord = oldTreeDataHash[key];

					newRecord = getObjectWithoutChildren(newRecord); //remove Children to avoid deep comparison
					oldRecord = getObjectWithoutChildren(oldRecord); //remove Children to avoid deep comparison

					if (newRecord && !angular.equals(newRecord, oldRecord)) {
						hasChange = performRowAction("updateRow", newRecord, hasChange);
					} else if (!newRecord) {
						hasChange = performRowAction("deleteRow", oldRecord, hasChange);
					}
				}

				for (var key in treeDataHash) {
					var newRecord = treeDataHash[key];
					if (!oldTreeDataHash.hasOwnProperty(key)) {
						newRecord = getObjectWithoutChildren(newRecord); //remove Children to avoid deep comparison

						if (newRecord.Id == newRecord.ParentId) {
							hasChange = performRowAction("addRow", newRecord, hasChange);
						} else {
							hasChange = performRowAction("addChild", newRecord, hasChange, newRecord.ParentId);
						}
					}
				}

				if (hasChange) {
					updateFilterAndSortState();
				}
			}

			function performRowAction(action, item, hasChange, parentId) {

				if (parentId) {
					hasChange = true;
					gridElement.igTreeGridUpdating(action, item, parentId);

				} else {
					hasChange = true;
					gridElement.igTreeGridUpdating(action, item.Id, item);
				}

				return hasChange;
			}

			function flattenTreeToHashMap(childArray, parentId, hashMap) {

				if (childArray) {
					childArray.forEach(function(childNode) {
						hashMap[childNode.Id] = childNode;
						hashMap[childNode.Id]["ParentId"] = parentId;
						flattenTreeToHashMap(childNode.Children, childNode.Id, hashMap);
					});
				}
			}
		}
	};
});